// by Ty Rayner

out vec4			fragColor;
in vec2				texcoord;
in vec3				normal;

uniform bool		drawClouds;
uniform bool		drawDayNight;
uniform float		cloudsRot;
uniform vec3		lightv;

uniform sampler2D	earthDayTex;
uniform sampler2D	earthNightTex;
uniform sampler2D	earthCloudsTex;

vec3 rgb2hsv(vec3 c)
{
	vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
	vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
	vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
	
	float d = q.x - min(q.w, q.y);
	float e = 1.0e-10;
	return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

/*
vec3 hsv2rgb(vec3 c)
{
	vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
	vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
	return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}
 */

void main()
{
	//float M_PI = 3.14159265358979323846264;
	
	// Color constants
	vec4 k = vec4(0,0,0,1);
	vec4 w = vec4(1,1,1,1);
	vec4 c = vec4(0,0,0,0);
	vec4 nightClouds = w;
	
	vec4 outColor;
	
	// Rotate our clouds coordinate according to uniform cloudsRot
	vec2 texCoordClouds;
	texCoordClouds = texcoord;
	texCoordClouds[0] = texCoordClouds[0] + .5;
	
	vec4 texColor_day = texture(earthDayTex, texcoord);
	vec4 texColor_day_gamma;
	vec4 texColor_night;
	vec4 texColor_clouds;
	float reflectivity = 0;
	
	// Compute reflectivity
	vec3 rgb;
	rgb[0] = texColor_day[0];
	rgb[1] = texColor_day[1];
	rgb[2] = texColor_day[2];
	vec3 hsv = rgb2hsv(rgb);
	
	float valSmoothed0;
	float valSmoothed1;
	
	valSmoothed0 = smoothstep(.78, .82, hsv[2]);
	valSmoothed1 = 1. - smoothstep(.12, .15, hsv[2]);
	
	reflectivity = clamp(valSmoothed0 + valSmoothed1, 0, 1);
	
	
	// Brighten day texture
	texColor_day_gamma = pow(texColor_day, vec4(.7,.7,.7,1));
	
	
	// Compute specular
	float light = dot(lightv, normal);
	float light_mix;
	float specular;
	
	light_mix = smoothstep(-.3, .3, light);
	specular = smoothstep(.96, 1.0, light) * reflectivity;
	
	outColor = texColor_day_gamma;
	
	
	
	if (drawDayNight) {
		nightClouds = vec4(.1,.1,.27,1);
		texColor_night = texture(earthNightTex, texcoord);
		outColor = mix(texColor_night, outColor, light_mix);
	}
	outColor = pow(outColor, vec4(1-(.84*specular),1-(.8*specular),1-(.7*specular),1));
	
	
	
	if (drawClouds) {
		// Add clouds
		texColor_clouds = texture(earthCloudsTex, texCoordClouds);
		vec4 cloudColor = mix(nightClouds, w, light_mix);
		outColor = mix(outColor, cloudColor, texColor_clouds[0]);
	}
	
	
	fragColor = outColor;
}
